# -*- coding: utf-8 -*
#
# Copyright (C) 2010-2018  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

import datetime
from logging import error
import tempfile

import mock
from mock import call
from nose.tools import assert_equal  # @UnresolvedImport pylint: disable=E0611
from omniORB import CORBA
import cherrypy
try:
    import ldap
except ImportError:
    error("Could not import ldap, some test will probably fail...")

import twill.commands
from wsgi_intercept import add_wsgi_intercept

try:
    from fred_webadmin.auth import ldap_auth, corba_auth
except Exception:  # pylint: disable=W0703
    error("Could not import auth module, some test will probably fail...")

from fred_webadmin.tests.webadmin.base import DaphneTestCase, init_test_server, deinit_test_server
import fred_webadmin.controller.adif
import fred_webadmin.webwidgets.forms

from fred_webadmin.corba import Registry, ccReg
from fred_webadmin.webwidgets.forms.filterforms import DomainFilterForm
import pylogger.dummylogger as logger
import fred_webadmin.user as fred_webadmin_user


def setup_module():
    init_test_server()


def teardown_module():
    deinit_test_server()


class BaseADIFTestCase(DaphneTestCase):
    def setUp(self):
        super(BaseADIFTestCase, self).setUp()
        self.admin_mock = AdminMock()
        self.web_session_mock['Admin'] = self.admin_mock
        self.web_session_mock['user'] = fred_webadmin_user.User(UserMock())
        self.file_mgr_mock = mock.Mock(ccReg._objref_FileManager)  # pylint: disable=W0212
        self.web_session_mock['FileManager'] = self.file_mgr_mock
        self.logger_mock = mock.Mock(ccReg._objref_Logger)  # pylint: disable=W0212
        self.web_session_mock['Logger'] = self.logger_mock
        self.corba_conn_mock = CorbaConnectionMock(admin=self.admin_mock)
        self.monkey_patch(fred_webadmin.controller.adif, 'corba_obj', self.corba_conn_mock)


# pylint: disable=W0613
class GroupManagerMock(object):
    def getGroups(self):
        return [
            Registry.Registrar.Group.GroupData(
                1, "test_group_1", ccReg.DateType(0, 0, 0)),
            Registry.Registrar.Group.GroupData(
                10, "test_group_2", ccReg.DateType(20, 10, 2009)),
            Registry.Registrar.Group.GroupData(
                7, "test_group_3", ccReg.DateType(0, 0, 0))]

    def getMembershipsByRegistar(self, reg_id):
        return []

    def deleteGroup(self, group_id):
        pass

    def addRegistrarToGroup(self, reg_id, group_id):
        pass

    def removeRegistrarFromGroup(self, reg_id, group_id):
        pass

    def updateGroup(self, group_id, name):
        pass


class CertificationManagerMock(object):
    def __init__(self):
        super(CertificationManagerMock, self).__init__()

    def getCertificationsByRegistrar(self, reg_id):
        return []
        # [Registry.Registrar.Certification.CertificationData(
        #    1, ccReg.DateType(1, 1, 2008), ccReg.DateType(1, 1, 2010), 2, 17)]

    def createCertification(self, reg_id, from_date, to_date, score, file_id):
        return 17

    def updateCertification(self, crt_id, score, file_id):
        pass

    def shortenCertification(self, crt_id, to_date):
        pass


class AdminMock(object):
    def __init__(self):
        super(AdminMock, self).__init__()
        self.session = None
        self.group_manager_mock = GroupManagerMock()  # helper reference to GrouManagerMock
        self.certification_manager_mock = CertificationManagerMock()

    def getCountryDescList(self):
        return [Registry.CountryDesc(1, 'cz')]

    def getDefaultCountry(self):
        return 1

    def getGroupManager(self):
        return self.group_manager_mock

    def getCertificationManager(self):
        return self.certification_manager_mock

    def createSession(self, username):
        self.session = mock.Mock(spec=ccReg._objref_Session)  # pylint: disable=W0212
        self.session.getUser.return_value = UserMock()
        return "testSessionString"

    def getSession(self, session_str):
        return self.session

    def authenticateUser(self, user, pwd):
        pass

    def isRegistrarBlocked(self, reg_id):
        return False

    def getObjectStatusDescList(self, lang):
        return [
            Registry.ObjectStatusDesc(id=0, shortName='ok', name='Objekt is without restrictions'),
            Registry.ObjectStatusDesc(id=1, shortName='serverDeleteProhibited', name='Deletion forbidden'),
            Registry.ObjectStatusDesc(id=2, shortName='serverRenewProhibited', name='Registration renewal forbidden'),
            Registry.ObjectStatusDesc(id=3, shortName='serverTransferProhibited', name='Sponsoring registrar change forbidden'),
            Registry.ObjectStatusDesc(id=4, shortName='serverUpdateProhibited', name='Update forbidden'),
            Registry.ObjectStatusDesc(id=5, shortName='serverOutzoneManual', name='The domain is administratively kept out of zone'),
            Registry.ObjectStatusDesc(id=6, shortName='serverInzoneManual', name='The domain is administratively kept in zone'),
            Registry.ObjectStatusDesc(id=7, shortName='serverBlocked', name='Domain blocked'),
            Registry.ObjectStatusDesc(id=8, shortName='expirationWarning', name='The domain expires in 30 days'),
            Registry.ObjectStatusDesc(id=9, shortName='expired', name='Domain expired'),
            Registry.ObjectStatusDesc(id=10, shortName='unguarded', name='The domain is 30 days after expiration'),
            Registry.ObjectStatusDesc(id=11, shortName='validationWarning1', name='The domain validation expires in 30 days'),
            Registry.ObjectStatusDesc(id=12, shortName='validationWarning2', name='The domain validation expires in 15 days'),
            Registry.ObjectStatusDesc(id=13, shortName='notValidated', name='Domain not validated'),
            Registry.ObjectStatusDesc(id=14, shortName='nssetMissing', name="The domain doesn't have associated nsset"),
            Registry.ObjectStatusDesc(id=15, shortName='outzone', name="The domain isn't generated in the zone"),
            Registry.ObjectStatusDesc(id=16, shortName='linked', name='Has relation to other records in the registry'),
            Registry.ObjectStatusDesc(id=17, shortName='deleteCandidate', name='To be deleted'),
            Registry.ObjectStatusDesc(id=18, shortName='serverRegistrantChangeProhibited', name='Registrant change forbidden'),
            Registry.ObjectStatusDesc(id=19, shortName='deleteWarning', name='The domain will be deleted in 11 days'),
            Registry.ObjectStatusDesc(id=20, shortName='outzoneUnguarded', name='The domain is out of zone after 30 days in expiration state'),
            Registry.ObjectStatusDesc(id=21, shortName='conditionallyIdentifiedContact', name='Contact is conditionally identified'),
            Registry.ObjectStatusDesc(id=22, shortName='identifiedContact', name='Contact is identified'),
            Registry.ObjectStatusDesc(id=23, shortName='validatedContact', name='Contact is validated'),
            Registry.ObjectStatusDesc(id=24, shortName='mojeidContact', name='MojeID contact'),
        ]


class CorbaConnectionMock(object):
    def __init__(self, admin=AdminMock(), logger_obj=logger.DummyLogger(), mailer=None, filemgr=None, messages=None,
                 blocking=None, verification=None, notification=None, record_statement=None):
        super(CorbaConnectionMock, self).__init__()
        self.obj = {
            "ccReg.Admin": admin,
            "ccReg.Logger": logger_obj,
            "ccReg.Mailer": mailer,
            "ccReg.FileManager": filemgr,
            "Registry.Messages": messages,
            "Registry.Administrative.Blocking": blocking,
            "Registry.AdminContactVerification.Server": verification,
            "Registry.Notification.Server": notification,
            "Registry.RecordStatement.Server": record_statement,
        }

    def getObject(self, obj1, obj2):
        return self.obj[obj2]

    def connect(self, user, pwd):
        pass


class UserMock(object):
    def __init__(self):
        super(UserMock, self).__init__()

    def _get_id(self):
        return "test_user_id"

    def _get_username(self):
        return "test_username"

    def _get_firstname(self):
        return "test_firstname"

    def _get_surname(self):
        return "test_surname"
# pylint: enable=W0613


class TestADIFAuthentication(BaseADIFTestCase):
    def test_login_valid_corba_auth(self):
        """ Login passes when using valid corba authentication.
        """
        fred_webadmin.config.auth_method = 'CORBA'
        # Replace fred_webadmin.controller.adif.auth module with CORBA
        # module.
        self.monkey_patch(
            fred_webadmin.controller.adif, 'auth', corba_auth)

        twill.commands.go("http://localhost:8080/login")
        twill.commands.fv(1, "login", "test")
        twill.commands.fv(1, "password", "test pwd")
        twill.commands.fv(1, "corba_server", "0")
        twill.commands.submit()
        twill.commands.url("http://localhost:8080/summary/")
        twill.commands.code(200)

#    def ignoretest_login_unicode_username(self):
#        """ Login passes when using valid corba authentication.
#            THIS IS BROKEN, probably because of strange way
#            mechanize (and twill that uses it) handles unicode strings.
#        """
#        fred_webadmin.config.auth_method = 'CORBA'
#        # Replace fred_webadmin.controller.adif.auth module with CORBA
#        # module.
#        self.monkey_patch(
#            fred_webadmin.controller.adif, 'auth', corba_auth)
#        self.corba_mock.ReplayAll()
#
#        twill.commands.go("http://localhost:8080/login")
#        twill.commands.fv(1, "login", u"ěščěšřéýí汉语unicode")
#        twill.commands.fv(1, "password", "test pwd")
#        twill.commands.fv(1, "corba_server", "0")
#        twill.commands.submit()
#        twill.commands.url("http://localhost:8080/summary/")
#        twill.commands.code(200)

    def test_login_invalid_corba_auth(self):
        """ Login fails when using invalid corba authentication.
        """
        fred_webadmin.config.auth_method = 'CORBA'
        self.monkey_patch(fred_webadmin.controller.adif, 'auth', corba_auth)

        with mock.patch.object(AdminMock, 'authenticateUser') as mocked_authenticateUser:
            mocked_authenticateUser.side_effect = ccReg.Admin.AuthFailed

            twill.commands.go("http://localhost:8080/login")
            twill.commands.fv(1, "login", "test")
            twill.commands.fv(1, "password", "test pwd")
            twill.commands.fv(1, "corba_server", "0")
            twill.commands.submit()
            twill.commands.url("http://localhost:8080/login/")
            twill.commands.code(403)

        mocked_authenticateUser.assert_called_once_with('test', 'test pwd')

    def test_double_login(self):
        """ Loging in when already loged in redirects to /summary.
        """
        self.web_session_mock['corbaSessionString'] = "test session string"
        twill.commands.go("http://localhost:8080/login/")
        twill.commands.code(200)
        twill.commands.url("http://localhost:8080/summary/")

    def test_login_invalid_form(self):
        """ Login fails when submitting invalid form.
        """
        twill.commands.go("http://localhost:8080/login/")
        twill.commands.fv(1, "login", "")
        twill.commands.fv(1, "password", "")
        twill.commands.code(200)
        # Check that we did not leave the login page.
        twill.commands.url("http://localhost:8080/login/")


class TestADIFAuthenticationLDAP(BaseADIFTestCase):
    def setUp(self):
        super(TestADIFAuthenticationLDAP, self).setUp()
        self.monkey_patch(fred_webadmin.controller.adif, 'auth', ldap_auth)
        self.monkey_patch(fred_webadmin.config, 'auth_method', 'LDAP')
        self.monkey_patch(fred_webadmin.config, 'LDAP_scope', 'test ldap scope %s')
        self.monkey_patch(fred_webadmin.config, 'LDAP_server', 'test ldap server')
        # Mock out ldap.initialize method. We must not mock the whole ldap package,
        # because ldap_auth uses ldap exceptions.
        self.ldap_initialize_mock = mock.create_autospec(ldap.initialize)
        self.monkey_patch(ldap_auth.ldap, 'initialize', self.ldap_initialize_mock)

    def assert_ldap_called(self):
        assert_equal(self.ldap_initialize_mock.mock_calls, [call('test ldap server'),
                                                      call().simple_bind_s('test ldap scope test', 'test pwd')])

    def test_login_ldap_valid_credentials(self):
        """ Login passes when valid credentials are supplied when using LDAP.
        """
        twill.commands.go("http://localhost:8080/login")
        twill.commands.fv(1, "login", "test")
        twill.commands.fv(1, "password", "test pwd")
        twill.commands.fv(1, "corba_server", "0")
        twill.commands.submit()
        # Invalid credentials => stay at login page.
        twill.commands.url("http://localhost:8080/summary/")
        twill.commands.code(200)

        self.assert_ldap_called()

    def test_login_ldap_invalid_credentials(self):
        """ Login fails when invalid credentials are supplied when using LDAP.
        """
        self.ldap_initialize_mock.return_value.simple_bind_s.side_effect = ldap.INVALID_CREDENTIALS

        twill.commands.go("http://localhost:8080/login")
        twill.commands.fv(1, "login", "test")
        twill.commands.fv(1, "password", "test pwd")
        twill.commands.fv(1, "corba_server", "0")
        twill.commands.submit()
        # Invalid credentials => stay at login page.
        twill.commands.url("http://localhost:8080/login/")
        twill.commands.code(403)

        self.assert_ldap_called()

    def test_login_ldap_server_down(self):
        """ Login fails when using LDAP and LDAP server is down.
        """
        self.ldap_initialize_mock.return_value.simple_bind_s.side_effect = ldap.SERVER_DOWN

        twill.commands.go("http://localhost:8080/login")
        twill.commands.fv(1, "login", "test")
        twill.commands.fv(1, "password", "test pwd")
        twill.commands.fv(1, "corba_server", "0")
        twill.commands.submit()
        # Invalid credentials => stay at login page.
        twill.commands.url("http://localhost:8080/login/")

        self.assert_ldap_called()


class TestRegistrarBase(BaseADIFTestCase):
    def setUp(self):
        super(TestRegistrarBase, self).setUp()
        self.admin_mock.createSession('testuser')
        self.session_mock = self.admin_mock.getSession('testSessionString')
        # when called with argument 17, return this FileInfo:
        self.file_mgr_mock.info.side_effect = lambda fileinfo_id: \
            ccReg.FileInfo(1, 'testfile', 'testpath', 'testmime', 0, ccReg.DateType(10, 10, 2010), 100) \
            if fileinfo_id == 17 else None
        self.session_mock.updateRegistrar.side_effect = lambda reg: \
            42 if isinstance(reg, (ccReg.AdminRegistrar, Registry.Registrar.Detail)) else None
        self.session_mock.getDetail.side_effect = lambda ft_type, obj_id: \
            self._fabricate_registrar() if (ft_type, obj_id) == (ccReg.FT_REGISTRAR, 42) else None

    def _fabricate_registrar(self):
        """ Returns a fake Registrar object. """
        return (
            CORBA.Any(
                CORBA.TypeCode("IDL:Registry/Registrar/Detail:1.0"),
                Registry.Registrar.Detail(
                    id=42L, ico='', dic='', varSymb='', vat=True,
                    handle='test handle', name='Company l.t.d.',
                    organization='test org 1', street1='',
                    street2='', street3='', city='', stateorprovince='',
                    postalcode='', country='CZ', telephone='', fax='',
                    email='', url='www.nic.cz', credit='0.00',
                    unspec_credit=u'120.00',
                    access=[Registry.Registrar.EPPAccess(
                        id=1,
                        password='123456789',
                        md5Cert='60:7E:DF:39:62:C3:9D:3C:EB:5A:87:80:C1:73:4F:99'),
                    Registry.Registrar.EPPAccess(
                        id=2,
                        password='passwd',
                        md5Cert='6A:AC:49:24:F8:32:1E:B7:A1:83:B5:D4:CB:74:29:98')],
                    zones=[
                        Registry.Registrar.ZoneAccess(
                            id=1L, name='0.2.4.e164.arpa', credit='0',
                            fromDate=ccReg.DateType(1, 1, 2007),
                            toDate=ccReg.DateType(0, 0, 0)),
                        Registry.Registrar.ZoneAccess(
                            id=2L, name='cz', credit='0',
                            fromDate=ccReg.DateType(1, 1, 2007),
                            toDate=ccReg.DateType(0, 0, 0))], hidden=False)))


class TestRegistrar(TestRegistrarBase):
    def test_edit_correct_args(self):
        """ Registrar editation passes. """
        twill.commands.go("http://localhost:8080/registrar/edit/?id=42")
        twill.commands.fv(2, "handle", "test handle")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
        twill.commands.find("test handle")

    def test_edit_incorrect_zone_date_arg(self):
        """ Registrar editation does not pass when invalid zone date
            provided. """
        twill.commands.go("http://localhost:8080/registrar/edit/?id=42")

        twill.commands.fv(2, "handle", "test handle")
        twill.commands.fv(2, "zones-0-toDate", "test invalid date")
        twill.commands.submit()

        # Test that we stay in edit, because the form is not valid.
        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/edit/\?id=42")

    def test_create(self):
        """ Registrar creation passes.
        """
        # Create the registrar.
        twill.commands.go("http://localhost:8080/registrar/create")
        twill.commands.fv(2, "handle", "test handle")
        twill.commands.submit()

        # Test that we've jumped to the detail page (i.e., creation has
        # completed successfully).
        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
        twill.commands.find("test handle")

    def test_create_registrar_zone_to_date_smaller_than_zone_from_date(self):
        """ Registrar creation fails when zone 'To' date is smaller than zone
            'From' date (ticket #3530)."""
        # Create the registrar.
        twill.commands.go("http://localhost:8080/registrar/create")
        twill.commands.fv(2, "handle", "test handle")
        # Fill in the zone name (mandatory field).
        twill.commands.fv(2, "zones-0-name", "test zone")
        # 'To' date is smaller than 'From' date.
        twill.commands.fv(2, "zones-0-fromDate", "2010-02-01")
        twill.commands.fv(2, "zones-0-toDate", "2010-01-01")
        twill.commands.submit()

        # Check that we're still at the 'create' page (i.e., the registrar has
        # not been created.
        twill.commands.code(200)
        twill.commands.url("http://localhost:8080/registrar/create")
        twill.commands.find('must be bigger')

    def test_create_registrar_zone_to_date_bigger_than_zone_from_date(self):
        """ Registrar creation passes when zone 'To' date is bigger than zone
            'From' date."""
        # Create the registrar.
        twill.commands.go("http://localhost:8080/registrar/create")
        twill.commands.fv(2, "handle", "test handle")
        # Fill in the zone name (mandatory field).
        twill.commands.fv(2, "zones-0-name", "test zone")
        # 'To' date is bigger than 'From' date.
        twill.commands.fv(2, "zones-0-fromDate", datetime.date.today().isoformat())
        twill.commands.fv(2, "zones-0-toDate", (datetime.date.today() + datetime.timedelta(7)).isoformat())
        twill.commands.submit()

        # Test that we've jumped to the detail page (i.e., creation has
        # completed successfully).
        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
        twill.commands.find("test handle")

    def test_create_two_registrars_with_same_name(self):
        """ Creating second registrar with the same name fails.
            Ticket #3079. """
        # Create the first registrar.
        twill.commands.go("http://localhost:8080/registrar/create")
        twill.commands.fv(2, "handle", "test handle")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
        twill.commands.find("test handle")

        # Now create the second one with the same name.
        self.session_mock.updateRegistrar.side_effect = ccReg.Admin.UpdateFailed

        twill.commands.go("http://localhost:8080/registrar/create")
        twill.commands.fv(2, "handle", "test handle")
        twill.commands.submit()

        # Test that we've stayed at the 'create' page (i.e., creation has
        # failed).
        twill.commands.url("http://localhost:8080/registrar/create")
        twill.commands.code(200)
        twill.commands.find('Updating registrar failed')


class TestRegistrarGroups(TestRegistrarBase):
    def test_add_registrar_to_group(self):
        twill.commands.go("http://localhost:8080/registrar/edit/?id=42")

        self.monkey_patch(self.admin_mock.group_manager_mock, 'getMembershipsByRegistar',
            lambda reg_id: [Registry.Registrar.Group.MembershipByRegistrar(1, 1,
                            ccReg.DateType(1, 1, 2008), ccReg.DateType(0, 0, 0))])

        twill.commands.fv(2, "groups-0-id", "1")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.find("test_group_1")

    def test_remove_registrar_from_group(self):
        with mock.patch.object(self.admin_mock.group_manager_mock, 'getMembershipsByRegistar') as mock_getMembershipsByReg:
            mock_getMembershipsByReg.side_effect = \
                lambda reg_id: [Registry.Registrar.Group.MembershipByRegistrar(1, 1,
                                ccReg.DateType(1, 1, 2008), ccReg.DateType(0, 0, 0))]

            twill.commands.go("http://localhost:8080/registrar/edit/?id=42")

        # now with normal getMembershipsByRegistar method which returns empty list:

        twill.commands.fv(2, "groups-0-DELETE", "1")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.notfind("test_group_1")


class TestRegistrarCertifications(TestRegistrarBase):
    def setUp(self):
        super(TestRegistrarCertifications, self).setUp()

    def test_add_certification(self):
        """ Correctly configured certification is added.
        """
        with tempfile.NamedTemporaryFile() as tmp_file:
            twill.commands.go("http://localhost:8080/registrar/edit/?id=42")
            twill.commands.fv(2, "certifications-0-fromDate", datetime.date.today().isoformat())
            twill.commands.fv(2, "certifications-0-toDate", (datetime.date.today() + datetime.timedelta(7)).isoformat())
            twill.commands.fv(2, "certifications-0-score", "2")
            twill.commands.formfile(2, "certifications-0-evaluation_file", tmp_file.name)

            file_upload_mock = mock.Mock()
            self.file_mgr_mock.save.side_effect = lambda name, mimetype, filetype: \
                file_upload_mock if tmp_file.name.endswith(name) \
                                     and filetype == 6 \
                                 else None
            file_upload_mock.finalize_upload.return_value = 17

            # Jump to detail after updating.
            get_cert_by_reg_mock = mock.Mock()
            self.monkey_patch(self.admin_mock.certification_manager_mock, 'getCertificationsByRegistrar',
                              get_cert_by_reg_mock)
            get_cert_by_reg_mock.side_effect = [
                [],  # first call return empty list
                [  # second call return CertificationData:
                    Registry.Registrar.Certification.CertificationData(
                        1, ccReg.DateType(1, 1, 2008),
                        ccReg.DateType(1, 1, 2010), 2, 17)
                ]
            ]
            twill.commands.submit()

            twill.commands.code(200)
            twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
            twill.commands.find(r'''<a href="/file/detail/\?id=17"''')

    def test_add_certification_no_file(self):
        """ Certification is not added when no file has been uploaded.
        """
        twill.commands.go("http://localhost:8080/registrar/edit/?id=42")
        twill.commands.fv(2, "certifications-0-fromDate", datetime.date.today().isoformat())
        twill.commands.fv(2, "certifications-0-toDate", (datetime.date.today() + datetime.timedelta(7)).isoformat())
        twill.commands.fv(2, "certifications-0-score", "2")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/edit/\?id=42")
        twill.commands.find('You have not specified the upload file for a certification')

    class DateMock(object):
        """ Mock class to replace datetime.date.today()
            (we do not want it to return the real current date).
        """
        @classmethod
        def today(cls):
            return datetime.date(2008, 1, 1)

    def test_shorten_certification(self):
        """ It is possible to shorten the certification.
        """
        date_mock = TestRegistrarCertifications.DateMock
        self.monkey_patch(fred_webadmin.webwidgets.forms.editforms, 'date', date_mock)

        get_cert_by_reg_mock = mock.Mock()
        result_1 = [Registry.Registrar.Certification.CertificationData(
                        1, ccReg.DateType(1, 1, 2008),
                        ccReg.DateType(1, 1, 2010), 2, 17)]
        # result 2 is same as 1, but must be here again, because it's converted in-place by corbarecoder
        result_2 = [Registry.Registrar.Certification.CertificationData(
                        1, ccReg.DateType(1, 1, 2008),
                        ccReg.DateType(1, 1, 2010), 2, 17)]
        result_3 = [Registry.Registrar.Certification.CertificationData(
                        1, ccReg.DateType(1, 1, 2008),
                        ccReg.DateType(12, 12, 2009), 2, 17)]
        get_cert_by_reg_mock.side_effect = [result_1, result_2, result_3]
        self.monkey_patch(self.admin_mock.certification_manager_mock, 'getCertificationsByRegistrar',
                          get_cert_by_reg_mock)

        twill.commands.go("http://localhost:8080/registrar/edit/?id=42")
        twill.commands.fv(2, "certifications-0-toDate", "2009-12-12")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.url(r"http://localhost:8080/registrar/detail/\?id=42")
        twill.commands.find(r'''<a href="/file/detail/\?id=17"''')
        twill.commands.find("2009-12-12")


class TestLoggerNoLogView(BaseADIFTestCase):
    def setUp(self):
        self.monkey_patch(fred_webadmin.config, 'debug', False)
        self.monkey_patch(fred_webadmin.config, 'auth_method', 'CORBA')
        audit_log = {
            'viewing_actions_enabled': False,
            'logging_actions_enabled': False,
            'force_critical_logging': False,
        }
        self.monkey_patch(fred_webadmin.config, 'audit_log', audit_log)
        super(TestLoggerNoLogView, self).setUp()

        root = fred_webadmin.controller.adif.prepare_root()
        wsgiApp = cherrypy.tree.mount(root)
        add_wsgi_intercept('localhost', 8080, lambda: wsgiApp)

    def tearDown(self):
        super(TestLoggerNoLogView, self).tearDown()
        # return back normal web root
        root = fred_webadmin.controller.adif.prepare_root()
        wsgiApp = cherrypy.tree.mount(root)
        add_wsgi_intercept('localhost', 8080, lambda: wsgiApp)

    def test_logger_hidden_when_log_view_is_disabled_in_config(self):
        # Replace fred_webadmin.controller.adif.auth module with CORBA
        # module.
        self.monkey_patch(fred_webadmin.controller.adif, 'auth', corba_auth)

        twill.commands.go("http://localhost:8080/login")
        twill.commands.fv(1, "login", "test")
        twill.commands.fv(1, "password", "test pwd")
        twill.commands.fv(1, "corba_server", "0")
        twill.commands.submit()

        twill.commands.go("http://localhost:8080/logger")
        twill.commands.url("http://localhost:8080/logger")
        # Test that the page has not been found.
        twill.commands.code(404)


class TestRegistrarGroupEditor(BaseADIFTestCase):
    def test_display_two_groups(self):
        """ Two registrar groups are displayed.
        """
        twill.commands.go("http://localhost:8080/group")
        twill.commands.code(200)

        twill.commands.find(
            '''<input title="test_group_1" type="text" name="groups-0-name"'''
            ''' value="test_group_1" />''')
        twill.commands.find(
            '''<input title="test_group_3" type="text" name="groups-1-name"'''
            ''' value="test_group_3" />''')

    def test_display_zero_groups(self):
        """ Two registrar groups are displayed.
        """
        self.monkey_patch(self.admin_mock.group_manager_mock, 'getGroups', lambda: [])

        twill.commands.go("http://localhost:8080/group")
        twill.commands.code(200)
        twill.commands.find(
            '''<input type="text" name="groups-0-name" value="" />''')

    def test_delete_group(self):
        """ Two registrar groups are displayed, one gets deleted.
        """
        self.monkey_patch(self.admin_mock.group_manager_mock, 'getGroups', lambda: \
            [Registry.Registrar.Group.GroupData(
                1, "test_group_1", ccReg.DateType(0, 0, 0)),
            Registry.Registrar.Group.GroupData(
                2, "test_group_2", ccReg.DateType(0, 0, 0))])

        twill.commands.go("http://localhost:8080/group")

        # simulate one group delete
        self.monkey_patch(self.admin_mock.group_manager_mock, 'getGroups', lambda: \
            [Registry.Registrar.Group.GroupData(
                2, "test_group_2", ccReg.DateType(0, 0, 0)),
            Registry.Registrar.Group.GroupData(
                1, "test_group_1", ccReg.DateType(20, 10, 2009))])

        twill.commands.code(200)
        twill.commands.fv(2, "groups-0-DELETE", "1")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.notfind(
            '''<input title="test_group_1" type="text" name="groups-0-name"'''
            ''' value="test_group_1" />''')
        twill.commands.find(
            '''<input title="test_group_2" type="text" name="groups-0-name"'''
            ''' value="test_group_2" />''')

    def test_delete_nonempty_group(self):
        """ Nonempty group cannot be deleted.
        """
        self.monkey_patch(self.admin_mock.group_manager_mock, 'getGroups', lambda: \
            [Registry.Registrar.Group.GroupData(
                1, "test_group_1", ccReg.DateType(0, 0, 0)),
            Registry.Registrar.Group.GroupData(
                2, "test_group_2", ccReg.DateType(0, 0, 0))])

        deleteGroup_mock = mock.Mock()
        deleteGroup_mock.side_effect = Registry.Registrar.InvalidValue("Test message that group is nonempty.")
        self.monkey_patch(self.admin_mock.group_manager_mock, 'deleteGroup', deleteGroup_mock)

        twill.commands.go("http://localhost:8080/group")
        twill.commands.code(200)
        twill.commands.fv(2, "groups-0-DELETE", "1")
        twill.commands.submit()

        twill.commands.code(200)
        twill.commands.find(
            '''<input title="test_group_1" type="text" name="groups-0-name"'''
            ''' value="test_group_1" />''')


class TestGeneratedJavascripts(BaseADIFTestCase):
    def test_filter_forms_javascript(self):
        self.web_session_mock['filterforms'] = [DomainFilterForm]
        twill.commands.go("http://localhost:8080/filter_forms_javascript.js")
        twill.commands.code(200)
        assert_equal(twill.browser.result.headers['Content-Type'], 'text/javascript;charset=utf-8')
        twill.commands.find('function createRowdomain')

    def test_service_actions(self):
        self.logger_mock.getServices.return_value = []
        twill.commands.go("http://localhost:8080/service_actions.js")
        twill.commands.code(200)
        assert_equal(twill.browser.result.headers['Content-Type'], 'text/javascript;charset=utf-8')
        twill.commands.find('function get_actions')
