#
# Copyright (C) 2016-2018  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.

import os
import twill.commands as tc

from mock import Mock, call
from nose.tools import assert_equal

from fred_webadmin.corba import Registry
from fred_webadmin.tests.webadmin.test_adif import BaseADIFTestCase
from fred_webadmin.tests.webadmin.base import enable_corba_comparison_decorator, init_test_server, deinit_test_server


def setup_module():
    init_test_server()


def teardown_module():
    deinit_test_server()


class TestImportNotifEmails(BaseADIFTestCase):
    def setUp(self):
        super(TestImportNotifEmails, self).setUp()
        self.admin_mock.createSession('testuser')
        self.session_mock = self.admin_mock.getSession('testSessionString')
        self.notif_mock = Mock(Registry.Notification._objref_Server)  # pylint: disable=W0212
        self.web_session_mock['Notification'] = self.notif_mock
        tc.go('http://localhost:8080/domain/import_notif_emails/')

    def test_file_not_uploaded(self):
        tc.fv(2, 'submit', '')
        tc.submit()
        tc.find('This field is required.')

    def test_empty_file_uploaded(self):
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/empty_file'))
        tc.submit()
        tc.find('Wrong file format.')

    def test_file_with_invalid_id(self):
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2016-09-16_invalid_id.csv'))
        tc.submit()
        tc.find('Invalid value in column Id: "asdf9". It must be a whole number.')

    @enable_corba_comparison_decorator(Registry.Notification.DomainEmail)
    def test_file_ok(self):
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2016-09-16_OK.csv'))
        tc.submit()
        assert_equal(
            self.notif_mock.mock_calls,
            [call.set_domain_outzone_unguarded_warning_emails([
                Registry.Notification.DomainEmail(domain_id=4, email='poks1@nic.cz'),
                Registry.Notification.DomainEmail(domain_id=9, email='pokus2@nic.cz'),
                Registry.Notification.DomainEmail(domain_id=9, email='pokus3@nic.cz')])]
        )
        tc.find(r'Emails have been successfully saved. \(2 domains, 3 emails\)')

    @enable_corba_comparison_decorator(Registry.Notification.DomainEmail)
    def test_file_invalid_email(self):
        self.notif_mock.set_domain_outzone_unguarded_warning_emails.side_effect = \
            Registry.Notification.DOMAIN_EMAIL_VALIDATION_ERROR(
                [Registry.Notification.DomainEmail(domain_id=4, email='pok@s1@nic.cz')]
            )
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2016-09-16_ERR.csv'))
        tc.submit()
        assert_equal(
            self.notif_mock.mock_calls,
            [call.set_domain_outzone_unguarded_warning_emails([
                Registry.Notification.DomainEmail(domain_id=4, email='pok@s1@nic.cz'),
                Registry.Notification.DomainEmail(domain_id=9, email='pokus2@nic.cz'),
                Registry.Notification.DomainEmail(domain_id=9, email='pokus3@nic.cz')])]
        )
        tc.find('The file contains these invalid emails: pok@s1@nic.cz')

    @enable_corba_comparison_decorator(Registry.Notification.DomainEmail)
    def test_file_missing_column_email(self):
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2016-09-16_missing_column_email.csv'))
        tc.submit()
        tc.find('Missing column "Email list" on the row 3.')

    @enable_corba_comparison_decorator(Registry.Notification.DomainEmail)
    def test_file_missing_columns(self):
        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2016-09-16_missing_column_id.csv'))
        tc.submit()
        tc.find('Missing column "Id" on the row 2.')

    def test_file_unicode_in_email(self):
        self.notif_mock.set_domain_outzone_unguarded_warning_emails.side_effect = \
            Registry.Notification.DOMAIN_EMAIL_VALIDATION_ERROR(
                [Registry.Notification.DomainEmail(domain_id=4, email='lukas.jaba@example.cz\xc2\xa0')]
            )

        tc.formfile(2, 'domains_emails', os.path.join(os.path.dirname(__file__),
                                                      'data/domain_2017-02-01_unicode_in_email.csv'))
        tc.submit()
        tc.find('The file contains these invalid emails: lukas.jaba@example.cz\xc2\xa0'.decode('utf-8'))
